# 
# This is a module designed to simplify writing Big Brother external scripts in PERL. 
# It's pretty ugly at this point, without an install or perldoc, but it will be
# cleaned up and completed shortly as time permits. For now, it needs to be placed
# in a directory available for your perl modules, most likely the BBHOME. Just by
# using this module, you will get all of the ENV settings needed from your bbdef.sh
# set into %ENV, and all of your %ENV set into local variables (I.E.:$BBHOME will
# be a reference to $ENV{BBHOME}, so you can use $BBHOME ect...
# A routine (BigBrother->Report($HostName,$function,$color,$status)) is supplied
# that performs the command to tell Big Brother the current status.
# An IsDown($string) is also supplied, that returns true if the file 
# $BBHOME/ext/down.cfg contains a line for $string,from,to that the current time
# falls between from and to. From and to are specified in the syntax of Date::Manip
# (see perldoc for it to see how to specify). 
# A few other routines are supplied, that I will doc soon, but it is more complicated
# than it really needed to be because I needed suport for a more complex BBHOST
# configureation. In particular, I needed to have oracle(sid,user=xxxx,pass=yyyy), and
# have it show in BB as host.sid, so don't be over critiacal, it will be cleaner soon. 
# I am also uploading the Oracle.pl that uses this module, and others soon.
#
# Joe Bryant
# HiUCImOn@Yahoo.com
#
# v1.0  Joe Bryant
# v1.1  Craig Cook
#        added bb-snmpexttab support
#        minor bugfixes
#        added debug features
#        did not handle "'s passed to it correctly

package BigBrother;
use Data::Dumper ; # for debug only
$DEBUG = 0 ; # change to 1 to see debug output

sub import {
	my ($caller_package)=caller;

	if (!exists $ENV{BBHOME}) {
		my ($work,$dir);				# we need local vars
		($work,$script)=$0=~/(.*?)\/?([^\/]*)$/;	# now strip out the dir and our name
		$work="$work/../";				# we assume a subdir of BBHOME
		chomp($dir=`pwd`);				# change to BBHOME
		chdir($work);					#   to get
		chomp($work=`pwd`);				#   the real dir name
		$ENV{BBHOME}=$work;				# now set BBHOME to something real
		chdir($dir);					# and retunr to our dir
	}

	$BBHOME=$ENV{BBHOME};

	if (!exists $ENV{BBTMP}) {				# only run if not set
		my ($var,$val);
		foreach $line (`cd $BBHOME;. etc/bbdef.sh;set`) {
			chomp($line);						# drop EOL
			$line =~ /^\s*(.*?)\s*=\s*(.*)/;
			#($var,$val)=/^\s*(.*?)\s*=\s*(.*)/;		# get var and value
			#print "var: $1, val: $2\n\n" ;
			$ENV{$1}=$2;				# and set
		}
	}

	foreach $env_key (keys %ENV) {
		next if ($env_key=~/^\s*$/sig);
		*{"${caller_package}::${env_key}"}=\$ENV{$env_key}; # make env avail to calling package
	}

	open(IN,"$ENV{BBHOSTS}") || warn "Could not open $ENV{BBHOSTS}.\n";	# open bb-hosts

	while ($host_line = <IN>) {				# read contents
		chomp ($host_line);					# trim EOL
		next if ($host_line =~ /^\s*#/);			# Skip comments
		next if ($host_line =~ /^\s*$/);			# Skip blank lines

		(@host_line_args)=split /\s+/, $host_line;	# Split into pieces

		$first_field  = shift @host_line_args ;
		next if $first_field !~ /\d+\.\d+\.\d+\.\d+/ ; # must be a bb directive (page, group etc)

		$ip    = $first_field ;
		$host  = shift @host_line_args ;
		$pound = shift @host_line_args ;
		$name  = lc($host);			# force lower case to make finding easier

		next unless ($pound eq '#');		# Skip if token 3 isn't a '#'

		foreach $parm (@host_line_args) {	# Process all the parms
			$bbhosts{"$name~$parm"}=$parm;	# and store as keys in %bbhosts "$name~$parm"
			$bbhost{"$name~$parm"}=$host;	# and store as keys in %bbhosts "$name~$parm"
			$bbhostsIP{"$ip~$parm"}=$parm;	# and store as keys in %bbhosts "$name~$parm"
		}
	}
	close(IN);					# and close


        warn Data::Dumper->Dump([\%bbhosts],["bbhosts"])     if $DEBUG ;
        warn Data::Dumper->Dump([\%bbhost],["bbhost"])       if $DEBUG ;
        warn Data::Dumper->Dump([\%bbhostsIP],["bbhostsIP"]) if $DEBUG ;


	foreach $key (keys %bbhosts) {			# We also need to parse parms
		if (($host,$function,$parms)=$key=~/^(.*)~(\w+)\((.*)\)/) {
		} else {
			($host,$function)=$key=~/^(.*)~(.*)/;	# so break it into host, function
			$parms=$function;
		}
		$name=$bbhost{"$key"};					# and retrieve the name
		%temp=();						# clear the work hash
		@positional=();
		foreach (split(/,/,$parms)) {				# split up and process each parm
			if (/=/) {					# two choices, positional or keyword
				($var,$val)=split(/=/);			# split it on the on the '='
				$var=lc($var);				# store as lower case to be sure it's unique
				$temp{$var}=$val;			# and save for when it is needed
			} else {					
				push @positional,$_;				# it is positional
			}
		}
		if (@positional) {
		} else {
			$positional[0]=$function;
		}
		$positional=join(':',@positional);
		$bbitems{"$bbhost{$key}.$positional[0]"}=$function;
		$positional{"$function"}.="$positional ";
		foreach (keys %temp) {
			$parms{"$positional~$function~$_"}=$temp{$_};
		}
	}
	$hosts=join(' ',keys %host);
	use Date::Manip qw(ParseDate DateCalc Date_Cmp);
	if (-r "$BBHOME/ext/down.cfg") {
		open(DOWN,"$BBHOME/ext/down.cfg");
		$now=ParseDate("now");
		foreach (<DOWN>) {
			chomp;
			s/#.*//sig;
			next if /^\s*$/sig;
			($name,$duration,$when)=split(',');
			$key="$name~".++$index{$name};
			$fromtime{$key}=ParseDate($when);
			$tilltime{$key}=DateCalc($when,$duration);
		}
	}


  $ENV{SNMPEXTTAB_FILE} = "$BBHOME/etc/bb-snmpexttab" unless $ENV{SNMPEXTTAB_FILE} ;
  $SNMPEXTTAB_FILE = $ENV{SNMPEXTTAB_FILE} ;
  if ( -f "$SNMPEXTTAB_FILE" ) {
    open (SNMPEXTTAB, "$SNMPEXTTAB_FILE") || die "Could not open $SNMPEXTTAB_FILE.\n" ;

    while ($snmp_line = <SNMPEXTTAB>) {

      chomp ($snmp_line) ;

      next if ($snmp_line =~ /^\s*#/ ) ; # skip comment lines
      next if ($snmp_line =~ /^\s*$/ ) ; # skip blank lines

      ($hostname, $COMMUNITY, $rest) = split /\s+/, $snmp_line, 3 ;

      $hostname = lc ($hostname) ; # lower case, make searching easier

      $Hash_of_hosts{$hostname} = $COMMUNITY ;

      if ( defined $rest ) {
        @tests = split ' ', $rest ;

        # add to a hash pointing to an array of tests for that host
        #$Hash_of_tests{$hostname} = [ @tests ] ;
	foreach my $test (@tests) {
	  push @{$Hash_of_tests{$test}},  $hostname ;
	}
      }

    }

    warn Data::Dumper->Dump([\%Hash_of_tests],["Hash_of_tests"]) if $DFBUG;

  close SNMPEXTTAB;
  }

}

sub get_community_string {

  my ($package,$hostname_to_look_for) = @_ ;
  my $found_host = "false" ;

  foreach my $hostname ( keys %Hash_of_hosts ) {
    warn "key: $hostname look: $hostname_to_look_for\n" if $DEBUG;
    if ($hostname_to_look_for eq $hostname) {
       $found_host = "true" ; # set flag
       $community = $Hash_of_hosts{$hostname} ;
    }

  }

  if ($found_host eq "true" ) {
    return "$community" ;
  } else {
    return "public" ;
  };
}

sub get_hosts_for_this_test {

  my ($package,$test_to_look_for) = @_ ;
  my $found_test = "false" ;

  foreach my $test ( keys %Hash_of_tests ) {
    warn "value: $test look: $test_to_look_for\n" if $DEBUG;
    if ($test_to_look_for eq $test) {
      $found_test = "true" ; # set flag
      push @hosts_with_this_test , @{$Hash_of_tests{$test}} ;
      $all_tests=join(' ',@hosts_with_this_test);
      return $all_tests ;
    }

  }

      return 0 ;
}

sub IsDown {
	my ($package,$key)=@_;
	foreach $temp (grep /^$key~/,keys %fromtime) {
		if ((Date_Cmp($fromtime{$temp},$now)<0) and (Date_Cmp($tilltime{$temp},$now)>0)) {
			return 1;
		}
	}
	return 0;
} 

sub Positional {
	($package,$forkey)=@_;
	my $hosts='';
	foreach (grep(/^$forkey$/,keys %positional)) {
		$hosts.=$positional{$_}.' ';
	}
	return $hosts;
}

sub Parms {
	my ($package,$key,$default)=@_;
	if (exists $parms{$key}) {
		return $parms{$key};
	} else {
		return $default;
	}
}

sub Items {
	my ($package,$forkey)=@_;
	my $items='';
	if ($forkey) {
		foreach (keys %bbitems) {
			next unless $bbitems{$_}=~/^$forkey$/;
			push @items,$_;
		}
		$items=join(' ',@items);
	} else {
		$items=join(' ',keys %bbitems);
	}
	return $items;
}

sub Report {
        my($package,$HostName, $test, $color, $status) = @_ ;
        ($test)=split(/\./,$test);
        # Substitute dots by commas in the host name
        $HostName =~ s/\./,/g;
	$status =~ s/\\"//g ;  # remove any \" in status
	$status =~ s/"/\\"/g ; # change any " to \" in status
        # Build the command to report to Big Brother
        $color=lc($color);
        my $MyCmd= "$ENV{BB} $ENV{BBDISP} ".'"'."status $HostName.$test $color ".localtime(time)."\n".$status.'"';

        # For debugging purposes
        # print "$MyCmd\n";

        # Execute the command.
        `$MyCmd`;
}

1;

